@@ -6,6 +6,9 @@ from time import sleep
 import copy
 import platform
 import subprocess
+import pickle
+
+print("\nQuykHtml is currently using pickle to load and save templates.\nBe aware of templates you are loading, see their warnings on it.\nhttps://docs.python.org/3/library/pickle.html\n")
 
 
 class qhtml:
@@ -24,12 +27,13 @@ class qhtml:
         self.bootstrap = self.bootstrap()
         self.preview = self.new("div")
         self.last = None
+        # why do i have to do this self self
+        self.templates = self._templates(self)
 
     # Returns a new object of an html element
     # returns: Object
 
     def new(self, _type, _p=0):
-
         _split = _type.split(" ")
         _obj = ""
         if len(_split) > 1:
@@ -516,6 +520,19 @@ class qhtml:
             # pyperclip.copy(self.html())
             return self
 
+        def on_click_goto(self, url_to_nav_to, new_tab=True, no_https=False):
+            if no_https is False:
+                if new_tab:
+                    self.on_click("window.open('https://" + url_to_nav_to + "');")
+                else:
+                    self.on_click("window.location.href='https://" + url_to_nav_to + "';")
+            else:
+                if new_tab:
+                    self.on_click("window.open('" + url_to_nav_to + "');")
+                else:
+                    self.on_click("window.location.href='" + url_to_nav_to + "';")
+            return self
+
         # action="upload.php" method="post" enctype="multipart/form-data"
         def set_form_options(self, action_php_call, method_get_or_post, enctype="multipart/form-data"):
             method = method_get_or_post
@@ -677,7 +694,7 @@ class qhtml:
             _r = _r + '  }'
             _r = _r + '};'
             if randomize_call_for_fresh_data:
-                _r = _r + 'r.open("' + _type + '", "' + _str_path + "?ran=" + str(_ran) + '", ' + _async + ');'
+                _r = _r + 'r.open("' + _type + '", "' + _str_path + '?ran=" + Math.random().toString().slice(2),' + _async + ');'
             else:
                 _r = _r + 'r.open("' + _type + '", "' + _str_path + '", ' + _async + ');'
             _r = _r + 'r.send();}'
@@ -695,6 +712,13 @@ class qhtml:
         def has_preview(self):
             return self._onclick_showpreview_html
 
+        def set_img_placeholder(self, place_holder_size=150):
+            if self.type != 'img':
+                print('qhtml object set_img_placeholder error.\nShould be used on img type, you used it on ' + self.type)
+                return False
+            self.set_img_src('https://via.placeholder.com/' + str(place_holder_size))
+            return self
+
         # CLASS style object
 
         class style_obj:
@@ -727,6 +751,46 @@ class qhtml:
                 self._style = self._style + _style
                 return self.parent
 
+            def align(self, left_center_right="center"):
+                self.append('text-align:' + left_center_right + ';')
+                # self._style = self._style + 'text-align:' + left_center_right + ';'
+                return self.parent
+
+            def bg_color(self, color='white'):
+                self.append('background-color:' + color + ';')
+                # self._style = self._style + 'background-color:' + color + ';'
+                return self.parent
+
+            def float(self, left_or_right="left"):
+                self.append('text-align:' + left_or_right + ';')
+                # self._style = self._style + 'text-align:' + left_or_right + ';'
+                return self.parent
+
+            def font_size(self, size="18px"):
+                self.append('font-size:' + size + ';')
+                # self._style = self._style + 'font-size:' + size + ';'
+                return self.parent
+
+            def font_color(self, color='black'):
+                self.append('color:' + color + ';')
+                # self._style += 'color:' + color + ';'
+                return self.parent
+
+            def height(self, height):
+                self.append('height:' + height + ';')
+                # self._style += 'height:' + height + ';'
+                return self.parent
+
+            def width(self, width):
+                self.append('width:' + width + ';')
+                # self._style += 'width:' + width + ';'
+                return self.parent
+
+            def hide(self, none_or_hidden='none'):
+                self.append('display:' + none_or_hidden + ';')
+                # self._style += 'display:' + none_or_hidden + ';'
+                return self.parent
+
     class table:
         def __init__(self, rows_or_file_path, columns_or_styling_dict=-1):
             self.__qhtml = qhtml()
@@ -985,3 +1049,57 @@ class qhtml:
                   'integrity="sha384-B4gt1jrGC7Jh4AgTPSdUtOBvfO8shuf57BaghqFfPlYxofvL8/KUEfYiJOMMV+rV" ' \
                   'crossorigin="anonymous"></script> '
             return bss
+
+    # TODO convert from pickle to no required library
+    #   dir(obj) and obj.__dict__ are helpers
+    class _templates:
+        def __init__(self, qhtml_instance):
+            self.parent = qhtml_instance
+            self.__path = os.getcwd() + '/Templates/'
+            self.__path2 = 'Templates/'
+
+        def __space_to_underscore(self, _str, vice_versa=False):
+            s = self
+            if vice_versa:
+                return _str.replace('_', ' ')
+            else:
+                return _str.replace(' ', '_')
+
+            return False
+
+        def save(self, template_name: str, qhtml_obj_list: list):
+            s = self
+            if template_name:
+                template_name = self.__space_to_underscore(template_name)
+                index = -1
+                p = self.__path + template_name + '/'
+                if not os.path.isdir(p):
+                    os.mkdir(p)
+                for el in qhtml_obj_list:
+                    index += 1
+                    if not os.path.isfile(self.__path2 + template_name + '/' + template_name + '_' + str(index) + '.pickle'):
+                        open('templates/' + template_name + '/' + template_name + '_' + str(index) + '.pickle', 'w').close()
+                    pickle.dump(el, file=open('templates/' + template_name + '/' + template_name + '_' + str(index) + '.pickle', 'wb'))
+                    print('Pickle saving - > ' + str(el))
+
+            return True
+
+        def load(self, template_name):
+            s = self
+            if template_name:
+                template_name = self.__space_to_underscore(template_name)
+                p = self.__path + template_name + '/'
+                p2 = self.__path2 + template_name + '/'
+                _templates = []
+                if os.path.isdir(p):
+                    index = -1
+                    for file in os.listdir(p):
+                        index += 1
+                        qhtml_obj = pickle.load(file=open(p2 + file, 'rb'))
+                        _templates.append(qhtml_obj)
+
+                if len(_templates) > 0:
+                    print('Pickles Loaded - > ' + str(_templates))
+                    return _templates
+
+            return False
